/*
 * CFile1.c
 *
 * Created: 3/7/2018 12:17:36 PM
 *  Author: d030721
 */
#include <avr/io.h>

#ifndef F_CPU
#define F_CPU 16000000UL
#endif

#ifndef BAUD
#define BAUD 9600
#endif

#include <util/setbaud.h>

#include "USART.h"

// Local function prototypes

static void
USART_INIT(void);


/*
 * USART initialization
 */
static void
USART_INIT(void)
{
    UBRR0H = UBRRH_VALUE;
    UBRR0L = UBRRL_VALUE;

    UCSR0C |= (1 << UCSZ00) | (1 << UCSZ01);    // Use 8-bit character sizes
    UCSR0B |= (1 << RXEN0) | (1 << TXEN0);      // Turn on the transmission and reception circuitry
}

void
XbeeUSART_init()
{
    USART_INIT();
}

/*
 * Send a byte through USART
 * This routine polls the USART data buffer register full flag until it indicates it is ready
 */
void
USART_vSendByte(char c)
{
    loop_until_bit_is_set(UCSR0A, UDRE0);       // Wait until port is free

    UDR0 = c;                                   // Start sending byte
}

/*
 * Blocking read from USART.
 */
unsigned char
USART_receive(void)
{
    loop_until_bit_is_set(UCSR0A, RXC0);        // Wait until data is available

    return UDR0;                                // Read the data.
}
