/*
 * GccApplication3.c
 *
 * Created: 11/16/2017 7:11:55 PM
 * Author : d030721
 */

#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "XBEE.h"
#include "xbee_frame.h"

// Constants


/*
 * Xbee.c
 * Osama Bin Tariq <bintariq.osama@gmail.com>
 */


/*
 * Send Message Function
 *
 * This function is passed a character array of an XBee API Packet, along with the
 * length of the packet. The checksum is calculated, then the start delimeter is transmitted,
 * followed by the length of the packet, then the data, and finally the checksum.
 */
void
XBee_TX_Request(uint16_t addr, uint8_t * data, int len)
{
    struct packet_s {
		uint8_t length[2];
        uint8_t api_id;
        uint8_t frame_id;
        uint8_t address[2];
        uint8_t options;              
        uint8_t data[len];
        uint8_t checksum;
    } packet;
    uint8_t *send = &packet;
    uint8_t i = 0;
    uint8_t sum = 0;
	uint16_t frame_data_length = len + 5; //len->data length, 5 is rest of packet in FRAME DATA

    // Prepare the packet contents.
    //
	packet.length[0] = (frame_data_length >> 8) & 0xFF;
    packet.length[1] = frame_data_length & 0xFF;
    packet.api_id = 0x01;
    packet.frame_id = 0x00;
    packet.address[0] = (addr >> 8) & 0xFF;
    packet.address[1] = addr & 0xFF;
    packet.options = 0x01;      
    for (i = 0; i < len; i++) {
        uint8_t d = data[i];

        packet.data[i] = d;
    }
	
	//To calculate checksum, exclude Length and Checksum Byte(excluding 3 bytes in total) from struct packet_s
	for (sum = 0, i = 2; i < sizeof(struct packet_s)-3; i++) {		
		uint8_t d = send[i];

		sum += d;
	}
    //Checksum is calculated by adding the data values together, and subtracting the
    //last 8 bits from 0xFF.
    packet.checksum = 0xFF - (sum & 0x00FF);

    // Send frame start
    //
    USART_vSendByte(FRAME_DELIMITER);

    // Send (escaped) packet contents.
    //
    for (i = 0; i < sizeof(struct packet_s); i++) {
        uint8_t d = send[i];

        if ((d == FRAME_DELIMITER) || (d == ESCAPE) || (d == XON) || (d == XOFF)) {
            USART_vSendByte(ESCAPE);
            d ^= 0x20;          // Escape current byte.
        }

        USART_vSendByte(d);     // Send the (escaped) byte.
    }
}
