/*
 * CFile1.c
 *
 * Created: 2/8/2018 2:28:49 PM
 *  Author: d030721
 */

#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "xbee_frame.h"
#include "XBEE_reception.h"

// Local function prototypes.
static void XBEE_empty_RX_buffer(void);

/*
 * pos == 0 -> start byte
 * pos == 1 -> length MSB
 * pos == 2 -> length LSB
 * pos == 3 -> API identifier
 * pos == 4 -> Source address MSB
 * pos == 5 -> Source address LSB
 * pos == 6 -> RSSI
 * pos == 7 -> Options
 * pos == 8 -> Data
 * pos == 9 -> checksum
 */


uint8_t
XBEE_rec(void)
{
    uint8_t b = 0;              // Current byte from Xbee
    uint8_t pos = 0;            // Current packet position
    uint8_t checksumTotal = 0;  // Checksum sum
    uint16_t data_frame_length = 0;     // Packet length
    uint8_t error = 0;          // Packet processing error
	uint8_t payload_data = 0;

    while (1) {
        checksumTotal = 0;
        pos = 0;
        error = 0;
        while (1) {
            b = USART_receive();

            if ((pos > 0) && (b == ESCAPE)) {
                b = USART_receive();
                b ^= 0x20;
            }

            if (pos >= 3)       // Checksum from API ID on
                checksumTotal += b;

            switch (pos) {
            case 0:
                if (b != FRAME_DELIMITER)
                    error = 1;

                break;

            case 1:
                data_frame_length = (b << 8);

                break;

            case 2:
                data_frame_length |= b & 0xFF;

                break;

            case 3:            // API ID
            case 4:            // Source address high
            case 5:            // Source address low
            case 6:            // RSSI
            case 7:            // Options
                break;

            case 8:            // FIXME: Valid ONLY for one data byte
                payload_data = b;

                break;

            default:
                // FIXME: Valid ONLY for one data byte
                //packet length does not include start, length, or checksum bytes, so add 3
                if ((pos != (data_frame_length + 3)) || ((checksumTotal & 0xff) != 0xff)) {
                    error = 1;

                    break;
                }

                // Done correct RX.
                return payload_data;    // FIXME: Valid ONLY for one data byte
            }

            if (error)
                break;          // Abort reading this packet.

            pos++;
        }
    }

    // Never reached
    return 0;
}
